==> adstax-sdk-scala_2.11-0.7.1-sources.jar: eu/shiftforward/adstax/ups/api/SingleAttributeMergingStrategy.scala <==
package eu.shiftforward.adstax.ups.api

import spray.json._

/**
 * A strategy to merge two attributes.
 */
sealed trait SingleAttributeMergingStrategy

/**
 * Attribute merging strategy that prefers the attribute value of the most recently changed user when merging
 * attributes.
 */
case object MostRecent extends SingleAttributeMergingStrategy

/**
 * Attribute merging strategy that sums both attribute values when merging attributes. When merging two attributes,
 * attribute values that are not [[NumericAttrValue]]s nor can't be directly converted to one are considered to be
 * [[NumericAttrValue]]s with value 0.
 *
 * [[StringAttrValue]]s whose value can be parsed to a double can be converted to a [[NumericAttrValue]].
 * [[BooleanAttrValue]]s with value `false` can be converted to a [[NumericAttrValue]] with value 0.
 * [[BooleanAttrValue]]s with value `true` can be converted to a [[NumericAttrValue]] with value 1.
 */
case object Sum extends SingleAttributeMergingStrategy

/**
 * Attribute merging strategy that uses the max of both attribute values as the final one. When merging two attributes,
 * attribute values that are not [[NumericAttrValue]]s nor can't be directly converted to one are considered to be
 * [[NumericAttrValue]]s with value `Double.MinValue`.
 *
 * [[StringAttrValue]]s whose value can be parsed to a double can be converted to a [[NumericAttrValue]].
 * [[BooleanAttrValue]]s with value `false` can be converted to a [[NumericAttrValue]] with value 0.
 * [[BooleanAttrValue]]s with value `true` can be converted to a [[NumericAttrValue]] with value 1.
 */
case object Max extends SingleAttributeMergingStrategy

/**
 * Attribute merging strategy that uses the min of both attribute values as the final one. When merging two attributes,
 * attribute values that are not [[NumericAttrValue]]s nor can't be directly converted to one are considered to be
 * [[NumericAttrValue]]s with value `Double.MaxValue`.
 *
 * [[StringAttrValue]]s whose value can be parsed to a double can be converted to a [[NumericAttrValue]].
 * [[BooleanAttrValue]]s with value `false` can be converted to a [[NumericAttrValue]] with value 0.
 * [[BooleanAttrValue]]s with value `true` can be converted to a [[NumericAttrValue]] with value 1.
 */
case object Min extends SingleAttributeMergingStrategy

/**
 * Attribute merging strategy that joins all the individual elements of both attribute values in a single array. When
 * merging two attributes, attribute values that are not [[ArrayAttrValue]]s are converted to an [[ArrayAttrValue]]
 * whose value is a sequence with a single element: the value of [[AttributeValue]] prior to the conversion.
 */
case object Concatenate extends SingleAttributeMergingStrategy

/**
 * Attribute merging strategy that merges the attribute values, provided that they are both maps. When conflicting keys
 * arise, this merging strategy preferes the value of the most recently changed user. When merging two attributes,
 * attribute values that are not [[MapAttrValue]]s are considered to be [[MapAttrValue]] with a value of `Map.empty`.
 */
case object MapMerge extends SingleAttributeMergingStrategy

/**
 * Attribute merging strategy that performs a boolean AND between both attribute values. When merging two attributes,
 * attribute values that are not [[BooleanAttrValue]]s nor can't be directly converted to one are considered to be
 * [[BooleanAttrValue]]s with value `true`.
 *
 * [[NumericAttrValue]]s with value different from `0` are converted to [[BooleanAttrValue]]s with value `true`.
 * [[NumericAttrValue]]s with value equal to `0` are converted to [[BooleanAttrValue]]s with value `false`.
 * [[StringAttrValue]]s with value `"true"` are converted to [[BooleanAttrValue]]s with value `true`.
 * [[StringAttrValue]]s with value `"false"` are converted to [[BooleanAttrValue]]s with value `false`.
 * [[StringAttrValue]]s whose value can be parsed to a Double follow the rules of the [[NumericAttrValue]] conversion
 * enumerated above.
 */
case object And extends SingleAttributeMergingStrategy

/**
 * Attribute merging strategy that performs a boolean OR between both attribute values. When merging two attributes,
 * attribute values that are not [[BooleanAttrValue]]s nor can't be directly converted to one are considered to be
 * [[BooleanAttrValue]]s with value `false`.
 *
 * [[NumericAttrValue]]s with value different from `0` are converted to [[BooleanAttrValue]]s with value `true`.
 * [[NumericAttrValue]]s with value equal to `0` are converted to [[BooleanAttrValue]]s with value `false`.
 * [[StringAttrValue]]s with value `"true"` are converted to [[BooleanAttrValue]]s with value `true`.
 * [[StringAttrValue]]s with value `"false"` are converted to [[BooleanAttrValue]]s with value `false`.
 * [[StringAttrValue]]s whose value can be parsed to a Double follow the rules of the [[NumericAttrValue]] conversion
 * enumerated above.
 */
case object Or extends SingleAttributeMergingStrategy

object SingleAttributeMergingStrategy {
  implicit object SingleAttributeMergingStrategyJsonFormat extends JsonFormat[SingleAttributeMergingStrategy] {
    def read(json: JsValue) = json match {
      case JsString("most-recent") => MostRecent
      case JsString("sum") => Sum
      case JsString("max") => Max
      case JsString("min") => Min
      case JsString("concatenate") => Concatenate
      case JsString("map-merge") => MapMerge
      case JsString("and") => And
      case JsString("or") => Or
      case strategy => deserializationError(s"Invalid single attribute merging strategy: ${strategy.compactPrint}")
    }

    def write(obj: SingleAttributeMergingStrategy) = obj match {
      case MostRecent => JsString("most-recent")
      case Sum => JsString("sum")
      case Max => JsString("max")
      case Min => JsString("min")
      case Concatenate => JsString("concatenate")
      case MapMerge => JsString("map-merge")
      case And => JsString("and")
      case Or => JsString("or")
    }
  }
}

==> adstax-sdk-scala_2.12-0.7.4-sources.jar: eu/shiftforward/adstax/ups/api/SingleAttributeMergingStrategy.scala <==
package eu.shiftforward.adstax.ups.api

import spray.json._

/**
 * A strategy to merge two attributes.
 */
sealed trait SingleAttributeMergingStrategy

/**
 * Attribute merging strategy that prefers the attribute value of the most recently changed user when merging
 * attributes.
 */
case object MostRecent extends SingleAttributeMergingStrategy

/**
 * Attribute merging strategy that prefers the attribute value of the least recently changed user when merging
 * attributes.
 */
case object Older extends SingleAttributeMergingStrategy

/**
 * Attribute merging strategy that sums both attribute values when merging attributes. When merging two attributes,
 * attribute values that are not [[NumericAttrValue]]s nor can't be directly converted to one are considered to be
 * [[NumericAttrValue]]s with value 0.
 *
 * [[StringAttrValue]]s whose value can be parsed to a double can be converted to a [[NumericAttrValue]].
 * [[BooleanAttrValue]]s with value `false` can be converted to a [[NumericAttrValue]] with value 0.
 * [[BooleanAttrValue]]s with value `true` can be converted to a [[NumericAttrValue]] with value 1.
 */
case object Sum extends SingleAttributeMergingStrategy

/**
 * Attribute merging strategy that uses the max of both attribute values as the final one. When merging two attributes,
 * attribute values that are not [[NumericAttrValue]]s nor can't be directly converted to one are considered to be
 * [[NumericAttrValue]]s with value `Double.MinValue`.
 *
 * [[StringAttrValue]]s whose value can be parsed to a double can be converted to a [[NumericAttrValue]].
 * [[BooleanAttrValue]]s with value `false` can be converted to a [[NumericAttrValue]] with value 0.
 * [[BooleanAttrValue]]s with value `true` can be converted to a [[NumericAttrValue]] with value 1.
 */
case object Max extends SingleAttributeMergingStrategy

/**
 * Attribute merging strategy that uses the min of both attribute values as the final one. When merging two attributes,
 * attribute values that are not [[NumericAttrValue]]s nor can't be directly converted to one are considered to be
 * [[NumericAttrValue]]s with value `Double.MaxValue`.
 *
 * [[StringAttrValue]]s whose value can be parsed to a double can be converted to a [[NumericAttrValue]].
 * [[BooleanAttrValue]]s with value `false` can be converted to a [[NumericAttrValue]] with value 0.
 * [[BooleanAttrValue]]s with value `true` can be converted to a [[NumericAttrValue]] with value 1.
 */
case object Min extends SingleAttributeMergingStrategy

/**
 * Attribute merging strategy that joins all the individual elements of both attribute values in a single array. When
 * merging two attributes, attribute values that are not [[ArrayAttrValue]]s are converted to an [[ArrayAttrValue]]
 * whose value is a sequence with a single element: the value of [[AttributeValue]] prior to the conversion.
 */
case object Concatenate extends SingleAttributeMergingStrategy

/**
 * Attribute merging strategy that merges the attribute values, provided that they are both maps. When conflicting keys
 * arise, this merging strategy preferes the value of the most recently changed user. When merging two attributes,
 * attribute values that are not [[MapAttrValue]]s are considered to be [[MapAttrValue]] with a value of `Map.empty`.
 */
case object MapMerge extends SingleAttributeMergingStrategy

/**
 * Attribute merging strategy that performs a boolean AND between both attribute values. When merging two attributes,
 * attribute values that are not [[BooleanAttrValue]]s nor can't be directly converted to one are considered to be
 * [[BooleanAttrValue]]s with value `true`.
 *
 * [[NumericAttrValue]]s with value different from `0` are converted to [[BooleanAttrValue]]s with value `true`.
 * [[NumericAttrValue]]s with value equal to `0` are converted to [[BooleanAttrValue]]s with value `false`.
 * [[StringAttrValue]]s with value `"true"` are converted to [[BooleanAttrValue]]s with value `true`.
 * [[StringAttrValue]]s with value `"false"` are converted to [[BooleanAttrValue]]s with value `false`.
 * [[StringAttrValue]]s whose value can be parsed to a Double follow the rules of the [[NumericAttrValue]] conversion
 * enumerated above.
 */
case object And extends SingleAttributeMergingStrategy

/**
 * Attribute merging strategy that performs a boolean OR between both attribute values. When merging two attributes,
 * attribute values that are not [[BooleanAttrValue]]s nor can't be directly converted to one are considered to be
 * [[BooleanAttrValue]]s with value `false`.
 *
 * [[NumericAttrValue]]s with value different from `0` are converted to [[BooleanAttrValue]]s with value `true`.
 * [[NumericAttrValue]]s with value equal to `0` are converted to [[BooleanAttrValue]]s with value `false`.
 * [[StringAttrValue]]s with value `"true"` are converted to [[BooleanAttrValue]]s with value `true`.
 * [[StringAttrValue]]s with value `"false"` are converted to [[BooleanAttrValue]]s with value `false`.
 * [[StringAttrValue]]s whose value can be parsed to a Double follow the rules of the [[NumericAttrValue]] conversion
 * enumerated above.
 */
case object Or extends SingleAttributeMergingStrategy

object SingleAttributeMergingStrategy {
  implicit object SingleAttributeMergingStrategyJsonFormat extends JsonFormat[SingleAttributeMergingStrategy] {
    def read(json: JsValue) = json match {
      case JsString("most-recent") => MostRecent
      case JsString("older") => Older
      case JsString("sum") => Sum
      case JsString("max") => Max
      case JsString("min") => Min
      case JsString("concatenate") => Concatenate
      case JsString("map-merge") => MapMerge
      case JsString("and") => And
      case JsString("or") => Or
      case strategy => deserializationError(s"Invalid single attribute merging strategy: ${strategy.compactPrint}")
    }

    def write(obj: SingleAttributeMergingStrategy) = obj match {
      case MostRecent => JsString("most-recent")
      case Older => JsString("older")
      case Sum => JsString("sum")
      case Max => JsString("max")
      case Min => JsString("min")
      case Concatenate => JsString("concatenate")
      case MapMerge => JsString("map-merge")
      case And => JsString("and")
      case Or => JsString("or")
    }
  }
}
